<?xml version="1.0" encoding="UTF-8"?>
<?bluebottle format version="0.1" ?>
<?xml-stylesheet type="text/xsl" href="http://bluebottle.ethz.ch/bluebottle.xsl" ?>
<Text>
<Span style="Normal"><![CDATA[MODULE GUIPatWc;]]></Span><Span style="Comment"><![CDATA[
	(** 	PURPOSE "Template for the construction of a GUI application which can be started multiple times.
					The template is a complete, correctly working, correctly terminating application";
		TECHNIQUE "The windows are chained form an anchor point windows. At cleanup time, the remaining chain
					elements are removed from the desktop";
		USAGE  "1 - Make a copy of this template and assign it a name of your choice";
				"2 - In the copy, replace all occurences of  "GUIPatWc" by the name of the new module.
					 Use the [Search] button in the tool bar to access the Search/Replace facility";
				"3 - Remove all comments and add comments about the new application to develop";
		AUTHOR "afi";
*)]]></Span><Span style="Normal"><![CDATA[

IMPORT
	AosModules, WMComponents, WM := WMWindowManager, WMStandardComponents,
	Utilities
	;

TYPE
	]]></Span><Span style="Bold"><![CDATA[Window* = OBJECT (WMComponents.FormWindow)]]></Span><Span style="Normal"><![CDATA[
	
		VAR next : Window;
	
		]]></Span><Span style="Bold"><![CDATA[PROCEDURE CreateForm () : WMComponents.VisualComponent;]]></Span><Span style="Normal"><![CDATA[
		VAR panel : WMStandardComponents.Panel;
		BEGIN
			NEW (panel); panel.bounds.SetExtents (600, 400); panel.fillColor.Set (0FFFFFFFFH);
			
			RETURN panel
		END CreateForm;

		]]></Span><Span style="Bold"><![CDATA[PROCEDURE &New ();
]]></Span><Span style="Normal"><![CDATA[		VAR vc : WMComponents.VisualComponent;]]></Span><Span style="Comment"><![CDATA[
]]></Span><Span style="Normal"><![CDATA[		BEGIN
			vc := CreateForm ();
			Init (vc.bounds.GetWidth (), vc.bounds.GetHeight (), FALSE);
			SetContent (vc);
			
			WM.DefaultAddWindow (SELF);
			SetTitle (Utilities.NewString ("GUIPatWc"));
			]]></Span><Span style="Comment"><![CDATA[(* Chain the previous window(s) to this new one, for guaranteed cleanup. *)]]></Span><Span style="Normal"><![CDATA[	(* A *)
	]]></Span><Span style="Lock"><![CDATA[		]]></Span><Span style="Normal"><![CDATA[next := windows;
	]]></Span><Span style="Lock"><![CDATA[	]]></Span><Span style="Normal"><![CDATA[	windows := SELF
		END New;

		]]></Span><Span style="Bold"><![CDATA[PROCEDURE Close;]]></Span><Span style="Normal"><![CDATA[
		BEGIN
			FreeWindow (SELF);		(* B *)
			Close^;
		END Close;

	END Window;

VAR
	windows : Window;

]]></Span><Span style="Comment"><![CDATA[(* Remove the window from the linked list *)
]]></Span><Span style="Bold"><![CDATA[PROCEDURE FreeWindow(free: Window);		(* C *)
]]></Span><Span style="Normal"><![CDATA[VAR
	winstance : Window;
BEGIN
	IF free = windows THEN]]></Span><Span style="Comment"><![CDATA[		(* the element to free is the first in list *)
]]></Span><Span style="Normal"><![CDATA[		windows := windows.next
	ELSE
		winstance := windows;
		WHILE (winstance # NIL) & (winstance.next # free) DO
			winstance := winstance.next
		END;
		IF winstance # NIL THEN]]></Span><Span style="Comment"><![CDATA[			(* not yet at the end of the chain: unchain it*)
]]></Span><Span style="Normal"><![CDATA[			winstance.next := free.next
		END
	END
END FreeWindow;
]]></Span><Span style="Bold"><![CDATA[
]]></Span><Span style="Comment"><![CDATA[(** Standard multiple windows controlled by chain, non-restorable. *)]]></Span><Span style="Bold"><![CDATA[
PROCEDURE Open* (par : ANY) : ANY;
]]></Span><Span style="Normal"><![CDATA[VAR winstance : Window;
BEGIN
	NEW (winstance);
	RETURN NIL
END Open;
]]></Span><Span style="Bold"><![CDATA[
PROCEDURE Cleanup;	(* D *)]]></Span><Span style="Normal"><![CDATA[
BEGIN
	WHILE windows # NIL DO
		windows.Close ();
	END
END Cleanup;

BEGIN
	AosModules.InstallTermHandler (Cleanup);

	]]></Span><Span style="Comment"><![CDATA[(* Additional BEGIN processing. *)]]></Span><Span style="Normal"><![CDATA[
END GUIPatWc.

GUIPatWc.Open ~
S.Free GUIPatWc ~]]></Span><Span style="Bold"><![CDATA[

Comments]]></Span><Span style="Normal"><![CDATA[

A.	Insert each new window instance at the beginning of a chain anchored on "windows".

B. and C.	Remove each closed window from the chain.

D.	Remove the remaining windows from the chain.]]></Span>

</Text>
